from ..base import _call

'''
All api requests under the configuration tab in https://developers.themoviedb.org/3/configuration
'''

def api():
    '''
    Get the system wide configuration information. Some elements of the API require some knowledge of this configuration data.
    The purpose of this is to try and keep the actual API responses as light as possible.
    It is recommended you cache this data within your application and check for updates every few days.

    This method currently holds the data relevant to building image URLs as well as the change key map.

    To build an image URL, you will need 3 pieces of data. The base_url, size and file_path.
    Simply combine them all and you will have a fully qualified URL. Here’s an example URL:
    https://image.tmdb.org/t/p/w500/8uO0gUM8aNqYLs1OsTBQiXu0fEv.jpg
    The configuration method also contains the list of change keys which can be useful if you are building an app that consumes data from the change feed.

    required: 
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/configuration')

def countries():
    '''
    Get the list of countries (ISO 3166-1 tags) used throughout TMDb.

    required: 
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/configuration/countries')

def jobs():
    '''
    Get a list of the jobs and departments we use on TMDb.

    required: 
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/configuration/jobs')

def languages():
    '''
    Get the list of languages (ISO 639-1 tags) used throughout TMDb.

    required: 
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/configuration/languages')

def primary_translations():
    '''
    Get a list of the officially supported translations on TMDb.

    While it's technically possible to add a translation in any one of the https://developers.themoviedb.org/3/configuration/get-languages we have added to TMDb (we don't restrict content),
    the ones listed in this method are the ones we also support for localizing the website with which means they are what we refer to as the "primary" translations.

    These are all specified as https://en.wikipedia.org/wiki/IETF_language_tag to identify the languages we use on TMDb. There is one exception which is image languages.
    They are currently only designated by a ISO-639-1 tag. This is a planned upgrade for the future.

    We're always open to adding more if you think one should be added. You can ask about getting a new primary translation added by posting on https://www.themoviedb.org/talk/category/5047951f760ee3318900009a.

    One more thing to mention, these are the translations that map to our website translation project. You can view and contribute to that project https://app.localeapp.com/projects/8267.

    required: 
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/configuration/primary_translations')

def timezones():
    '''
    Get the list of timezones used throughout TMDb.

    required: 
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/configuration/timezones')



from ..base import _call

'''
All api requests under the search tab in https://developers.themoviedb.org/3/search
'''

def companies(**kwargs):
    '''
    Search for companies.

    required: query
    optional: page
    '''

    return _call('GET', f'https://api.themoviedb.org/3/search/company', params=kwargs)

def collections(**kwargs):
    '''
    Search for collections.

    required: query
    optional: page, language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/search/collection', params=kwargs)

def keywords(**kwargs):
    '''
    Search for keywords.

    required: query
    optional: page
    '''

    return _call('GET', f'https://api.themoviedb.org/3/search/keyword', params=kwargs)

def movies(**kwargs):
    '''
    Search for movies.

    required: query
    optional: page, language, include_adult, region, year, primary_release_year
    '''

    return _call('GET', f'https://api.themoviedb.org/3/search/movie', params=kwargs)

def multi(**kwargs):
    '''
    Search multiple models in a single request.
    Multi search currently supports searching for movies, tv shows and people in a single request.

    required: query
    optional: page, language, include_adult, region
    '''

    return _call('GET', f'https://api.themoviedb.org/3/search/multi', params=kwargs)

def people(**kwargs):
    '''
    Search for people.

    required: query
    optional: page, language, include_adult, region
    '''

    return _call('GET', f'https://api.themoviedb.org/3/search/person', params=kwargs)

def tv(**kwargs):
    '''
    Search for a TV show.

    required: query
    optional: page, language, first_air_date_year
    '''

    return _call('GET', f'https://api.themoviedb.org/3/search/tv', params=kwargs)



from ..base import _call

'''
All api requests under the discover tab in https://developers.themoviedb.org/3/discover
'''


def movie(params=None, **kwargs):
    '''
    Discover movies by different types of data like average rating, number of votes, genres and certifications.
    You can get a valid list of certifications from the https://developers.themoviedb.org/3/certifications/get-movie-certifications method.

    Discover also supports a nice list of sort options. See below for all of the available options.

    Please note, when using certification/certification.lte you must also specify certification_country.
    These two parameters work together in order to filter the results.
    You can only filter results with the countries we have added to our https://developers.themoviedb.org/3/certifications/get-movie-certifications.

    If you specify the region parameter, the regional release date will be used instead of the primary release date.
    The date returned will be the first date based on your query (ie. if a with_release_type is specified).
    It's important to note the order of the release types that are used.
    Specifying "2|3" would return the limited theatrical release date as opposed to "3|2" which would return the theatrical date.

    Also note that a number of filters support being comma (,) or pipe (|) separated. Comma's are treated like an AND and query while pipe's are an OR.

    Some examples of what can be done with discover can be found https://www.themoviedb.org/documentation/api/discover.

    required:
    optional: language, region, sort_by, certification_country, certification, certification.lte, certification.gte, include_adult, include_video, page,
            primary_release_year, primary_release_date.gte, primary_release_date.lte, release_date.gte, release_date.lte,
            with_release_type, year, vote_count.gte, vote_count.lte, vote_average.gte, vote_average.lte, with_cast, with_crew,
            with_people, with_companies, with_genres, without_genres, with_keywords, without_keywords, with_runtime.gte, with_runtime.lte, with_original_language
    '''

    if params is None:
        params = kwargs
    return _call('GET', f'https://api.themoviedb.org/3/discover/movie', params=params)


def tv(params=None, **kwargs):
    '''
    Discover TV shows by different types of data like average rating, number of votes, genres, the network they aired on and air dates.

    Discover also supports a nice list of sort options. See below for all of the available options.

    Also note that a number of filters support being comma (,) or pipe (|) separated. Comma's are treated like an AND and query while pipe's are an OR.

    Some examples of what can be done with discover can be found https://www.themoviedb.org/documentation/api/discover.

    required:
    optional: language, sort_by, air_date.gte, air_date.lte, first_air_date.gte, first_air_date.lte, first_air_date_year,
            page, timezone, vote_count.gte, vote_count.lte, vote_average.gte, vote_average.lte, with_networks, with_companies, with_genres,
            without_genres, with_keywords, without_keywords, with_runtime.gte, with_runtime.lte, with_original_language, screened_theatrically, include_null_first_air_dates
    '''
    if params is None:
        params = kwargs
    return _call('GET', f'https://api.themoviedb.org/3/discover/tv', params=params)



from ..base import _call
'''
All api requests under the collections tab in https://developers.themoviedb.org/3/collections
'''

def details(collection_id, **kwargs):
    '''
    Get collection details by id.

    required: collection_id
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/collection/{collection_id}', params=kwargs)

def images(collection_id, **kwargs):
    '''
    Get the images for a collection by id.

    required: collection_id
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/collection/{collection_id}/images', params=kwargs)

def translations(collection_id, **kwargs):
    '''
    Get the list translations for a collection by id.

    required: collection_id
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/collection/{collection_id}/translations', params=kwargs)



from ..base import _call

'''
All api requests under the find tab in https://developers.themoviedb.org/3/find
'''

def id(external_id, **kwargs):
    '''
    The find method makes it easy to search for objects in our database by an external id. For example, an IMDB ID.
    This method will search all objects (movies, TV shows and people) and return the results in a single response.
    The supported external sources for each object are as follows.

    Media Databases
                   Movies       TV Shows        TV Seasons      TV Episodes         People
    IMDb ID        ✓            ✓               ✗               ✓                  ✓
    TVDB ID        ✗            ✓               ✓               ✓                  ✗
    Freebase MID*  ✗            ✓               ✓               ✓                  ✓
    Freebase ID*   ✗            ✓               ✓               ✓                  ✓
    TVRage ID*     ✗            ✓               ✓               ✓                  ✓


    Social IDs
                Movies      TV Shows        TV Seasons      TV Episodes         People
    Facebook     ✓          ✓               ✗               ✗                   ✓
    Instagram    ✓          ✓               ✗               ✗                   ✓
    Twitter      ✓          ✓               ✗               ✗                   ✓

    *Defunct or no longer available as a service.

    required: external_id, external_source
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/find/{external_id}', params=kwargs)



from ..base import _call

'''
All api requests under the movies tab in https://developers.themoviedb.org/3/movies
'''


def details(movie_id, **kwargs):
    '''
    Get the primary information about a movie.
    Supports append_to_response. https://developers.themoviedb.org/3/getting-started/append-to-response

    required: movie_id
    optional: language, append_to_response
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}', params=kwargs)


def account_states(movie_id, **kwargs):
    '''
    Grab the following account states for a session:
        Movie rating
        If it belongs to your watchlist
        If it belongs to your favourite list

    required: movie_id, session_id or guest_session_id
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/account_states', params=kwargs)


def alternative_titles(movie_id, **kwargs):
    '''
    Get all of the alternative titles for a movie.

    required: movie_id
    optional: country
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/alternative_titles', params=kwargs)


def changes(movie_id, **kwargs):
    '''
    Get the changes for a movie. By default only the last 24 hours are returned.
    You can query up to 14 days in a single query by using the start_date and end_date query parameters.

    required: movie_id
    optional: page, start_date, end_date
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/changes', params=kwargs)


def credits(movie_id):
    '''
    Get the cast and crew for a movie.

    required: movie_id
    optional: 
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/credits')


def external_ids(movie_id):
    '''
    Get the external ids for a movie. We currently support the following external sources.

    Media Databases: 
        IMDb ID 
    Social IDs:
        Facebook
        Instagram
        Twitter

    required: movie_id
    optional: 
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/external_ids')


def images(movie_id, **kwargs):
    '''
    Get the images that belong to a movie.
    Querying images with a language parameter will filter the results.
    If you want to include a fallback language (especially useful for backdrops) you can use the include_image_language parameter.
    This should be a comma seperated value like so: include_image_language=en,null.

    required: movie_id
    optional: language, include_image_language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/images', params=kwargs)


def keywords(movie_id):
    '''
    Get the keywords that have been added to a movie.

    required: movie_id
    optional: 
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/keywords')


def release_dates(movie_id):
    '''
    Get the release date along with the certification for a movie.
    Release dates support different types:
        Premiere
        Theatrical (limited)
        Theatrical
        Digital
        Physical
        TV

    required: movie_id
    optional: 
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/release_dates')


def videos(movie_id, **kwargs):
    '''
    Get the videos that have been added to a movie.

    required: movie_id
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/videos', params=kwargs)


def translations(movie_id):
    '''
    Get a list of translations that have been created for a movie.

    required: movie_id
    optional: 
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/translations')


def recommendations(movie_id, **kwargs):
    '''
    Get a list of recommended movies for a movie.

    required: movie_id
    optional: page, language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/recommendations', params=kwargs)


def similar(movie_id, **kwargs):
    '''
    Get a list of similar movies. This is not the same as the 'Recommendation' system you see on the website.
    These items are assembled by looking at keywords and genres.

    required: movie_id
    optional: page, language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/similar', params=kwargs)


def reviews(movie_id, **kwargs):
    '''
    Get the user reviews for a movie.

    required: movie_id
    optional: page, language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/reviews', params=kwargs)


def lists(movie_id, **kwargs):
    '''
    Get a list of lists that this movie belongs to.

    required: movie_id
    optional: page, language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/{movie_id}/lists', params=kwargs)


def latest(**kwargs):
    '''
    Get the most newly created movie. This is a live response and will continuously change.

    required: 
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/latest', params=kwargs)


def now_playing(**kwargs):
    '''
    Get a list of movies in theatres. This is a release type query that looks for all movies that have a release type of 2 or 3 within the specified date range.
    You can optionally specify a region prameter which will narrow the search to only look for theatrical release dates within the specified country.

    required: 
    optional: page, language, region
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/now_playing', params=kwargs)


def popular(**kwargs):
    '''
    Get a list of the current popular movies on TMDb. This list updates daily.

    required: 
    optional: page, language, region
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/popular', params=kwargs)


def top_rated(**kwargs):
    '''
    Get the top rated movies on TMDb.

    required: 
    optional: page, language, region
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/top_rated', params=kwargs)


def upcoming(**kwargs):
    '''
    Get a list of upcoming movies in theatres. This is a release type query that looks for all movies that have a release type of 2 or 3 within the specified date range.
    You can optionally specify a region prameter which will narrow the search to only look for theatrical release dates within the specified country.

    required: 
    optional: page, language, region
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/upcoming', params=kwargs)


def rate_movie(movie_id, rating, **kwargs):
    '''
    Rate a movie.
    A valid session or guest session ID is required. You can read more about how this works: https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id

    required: movie_id, rating, session_id or guest_session_id
    optional: 
    '''

    payload = {
        'value': rating
        }

    return _call('POST', f'https://api.themoviedb.org/3/movie/{movie_id}/rating', params=kwargs, payload=payload)


def delete_rating(movie_id, **kwargs):
    '''
    Remove your rating for a movie.
    A valid session or guest session ID is required. You can read more about how this works: https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id

    required: movie_id, session_id or guest_session_id
    optional: 
    '''

    return _call('DELETE', f'https://api.themoviedb.org/3/movie/{movie_id}/rating', params=kwargs)



from . import accounts as Accounts
from . import authentication as Authentication
from . import certifications as Certifications
from . import changes as Changes
from . import collections as Collections
from . import companies as Companies
from . import configuration as Configuration
from . import credits as Credits
from . import discover as Discover
from . import episode_groups as Episode_groups
from . import episodes as Episodes
from . import find as Find
from . import genres as Genres
from . import guests as Guests
from . import keywords as Keywords
from . import lists as Lists
from . import movies as Movies
from . import networks as Networks
from . import people as People
from . import reviews as Reviews
from . import search as Search
from . import seasons as Seasons
from . import trending as Trending
from . import tv as Tv
